﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Steema.TeeChart.Styles;

namespace MouseMove_Refresh
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            TeeChart.Axes.Right.Visible = true;
            TeeChart.Axes.Right.Labels.Visible = false;
            TeeChart.Axes.Top.Visible = true;
            TeeChart.Axes.Top.Labels.Visible = false;

            //Set the basic chart properties
            TeeChart.Axes.Bottom.Labels.Visible = true;

            TeeChart.Axes.Bottom.Labels.Font.Brush.Color = Color.Transparent;
            TeeChart.Axes.Bottom.Ticks.Visible = false;
            TeeChart.Axes.Bottom.TicksInner.Visible = true;
            TeeChart.Axes.Bottom.MinorTicks.Visible = false;
            TeeChart.Axes.Bottom.Grid.Visible = false;

            TeeChart.Axes.Left.Ticks.Visible = false;
            TeeChart.Axes.Left.TicksInner.Visible = true;
            TeeChart.Axes.Left.MinorTicks.Visible = false;
            TeeChart.Axes.Left.Grid.Visible = true;


            TeeChart.Panel.MarginLeft = 0;
            TeeChart.Panel.MarginRight = 0;
            TeeChart.Panel.MarginTop = 0;
            TeeChart.Panel.MarginBottom = 0;

            TeeChart.Header.Visible = false;
            TeeChart.Aspect.View3D = false;
            TeeChart.Legend.Visible = false;

            TeeChart.Dock = DockStyle.Fill;
            Random r = new Random();

						int numPoints = 10000;

            for (int i = 1; i <= 10; i++)
            {
								double[] Val = new double[numPoints];
								double[] xVal = new double[numPoints];

                FastLine _FastLine = new FastLine();
								for (int j = 0; j < numPoints; j++)
                {
                    int minval = 0;
                    int MaxVal = 0;
                    if (i == 0)
                    {
                        minval = 1;
                        MaxVal = 100;
                    }
                    else if (i <= 3)
                    {
                        minval = 300;
                        MaxVal = 600;
                    }
                    else if (i <= 6)
                    {
                        minval = 900;
                        MaxVal = 1500;
                    }
                    else
                    {
                        minval = 2000;
                        MaxVal = 2200;
                    }

                    Val[j] = r.Next(minval, MaxVal);
                    xVal[j] = (double)j;
                }

								_FastLine.YValues.Count = numPoints;
                _FastLine.YValues.Value = Val;
								_FastLine.XValues.Count = numPoints;
                _FastLine.XValues.Value = xVal;
                TeeChart.Series.Add(_FastLine);
            }
            //add data

        }

        #region TeeChart Events
        bool MouseDragInProgress = false;
        private void TeeChart_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                if (!IsCursurInChart(e)) return;

                MouseDragInProgress = true;
                CalculateNearestPoint(e);
                TeeChart.Invalidate();

            }
        }

        private void TeeChart_MouseMove(object sender, MouseEventArgs e)
        {
            if (TeeChart.Series.Count <= 0) return;
            if (!IsCursurInChart(e)) return;

            if (e.Button == MouseButtons.Right)
            {
                CalculateNearestPoint(e);
								DrawLine(TeeChart.Graphics3D);
								//TeeChart.Refresh();
            }
        }

        private void TeeChart_MouseUp(object sender, MouseEventArgs e)
        {
            MouseDragInProgress = false;

            //TODO:Cant call TeeChart.Refresh() function as it will take time to re draw the whole window
						//TeeChart.Refresh();
        }

        private void TeeChart_AfterDraw(object sender, Steema.TeeChart.Drawing.Graphics3D g)
        {

						//if (!MouseDragInProgress) return;
						//DrawLine(g);

        }

				private Point p0, p1;
				private Color color;

				private void DrawLine(Steema.TeeChart.Drawing.Graphics3D g)
				{
					try
					{
						miValueIndex = miValueIndex >= TeeChart.Series[miSeriesIndex].Count ? TeeChart.Series[miSeriesIndex].Count - 1 : miValueIndex < 0 ? 0 : miValueIndex;
						double xVal = TeeChart.Series[miSeriesIndex].XValues[miValueIndex];
						int xPosValue = TeeChart.Axes.Bottom.CalcXPosValue(xVal);
						int yPosValue;
						//Pen Style
						g.Brush.Visible = false;
						g.Pen.Style = System.Drawing.Drawing2D.DashStyle.Dash;

						int BottomPosVal = -1;
						string sValue = "";
						if (TeeChart.Series[miSeriesIndex] is Steema.TeeChart.Styles.FastLine)
						{
							if (TeeChart.Series.Count <= 0) return;

							Series _Series = TeeChart.Series[miSeriesIndex];
							sValue = _Series.YValues[(int)xVal].ToString("F2");
							yPosValue = _Series.GetVertAxis.CalcYPosValue(TeeChart.Series[miSeriesIndex].YValues[(int)xVal]);

							Rectangle chartRect = TeeChart.Chart.ChartRect;
							BottomPosVal = chartRect.Bottom;

							//Draw the line from the nearest point to the bottom axis
							g.Pen.Color = TeeChart.Series[miSeriesIndex].Color;
							//g.Line(xPosValue, BottomPosVal, xPosValue, yPosValue);

							DrawRectangle(p0, p1, color);
							p0 = new Point(xPosValue, BottomPosVal);
							p1 = new Point(xPosValue+1, yPosValue);
							color = g.Pen.Color;
							DrawRectangle(p0, p1, color);	

							//Draw the circle at the point
							g.Pen.Style = System.Drawing.Drawing2D.DashStyle.Solid;
							g.Brush.Color = Color.Gray;
							g.Brush.Transparency = 100;

							TeeChart.Canvas.Pen.Color = Color.Gray;
							TeeChart.Canvas.Pen.Width = 1;

							//Subtrating by radius of the circle
							//g.Ellipse(new Rectangle(xPosValue - 3, yPosValue - 3, 6, 6));
						}
					}
					catch (Exception ce)
					{
						System.Diagnostics.Debug.WriteLine(ce.Message);
						////TODO:Log the exception message
					}
				}

				private void DrawRectangle(Point p0, Point p1, Color color)
				{
					Rectangle r = new Rectangle(p0.X, p0.Y, p1.X - p0.X, p1.Y - p0.Y);
					ControlPaint.DrawReversibleFrame(ClipReversibleRectangle(r), color, FrameStyle.Dashed);
				}

				private Rectangle ClipReversibleRectangle(Rectangle r)
				{
					Point p = TeeChart.Chart.Parent.PointToScreen(new Point(0, 0));
									
					Rectangle origR = new Rectangle(p.X + r.Left, p.Y + r.Top, r.Width, r.Height);
					TeeChart.Graphics3D.OrientRectangle(ref origR);
					Rectangle chartR = TeeChart.Chart.ChartBounds;

					chartR.X = p.X;
					chartR.Y = p.Y;

					origR.Intersect(chartR);
					return origR;
				}

        #endregion

        #region Nearest Point Calculations
        //TODO: Need More work here
        private int GetNearestPointFromMouseLocation(Series _Series, int XPos, int YPos)
        {
					double XVal = _Series.XScreenToValue(XPos);
					int XActualVal = (int)Math.Round(XVal, 0);
					int yPosinCo = -1;
					if (XActualVal >= 0)
					{
						double Temp = _Series.YValues[XActualVal];
						yPosinCo = _Series.CalcYPosValue(Temp);
					}
					else
						return -1;
					return yPosinCo;
        }
        int miSeriesIndex, miValueIndex;
        private bool CalculateNearestPoint(MouseEventArgs e)
        {
            int[] NearestPoints;
            double Dist, tmp;
            Point P1, P2;

            NearestPoints = new int[TeeChart.Series.Count];
            P1 = new Point(0, 0);
            P2 = new Point(TeeChart.Width, TeeChart.Height);
            miSeriesIndex = 0;
            miValueIndex = 0;
            try
            {
                for (int i = 0; i < NearestPoints.Length; i++)
                {
                    Series s = TeeChart.Series[i];
                    NearestPoints[i] = GetNearestPointFromMouseLocation(s, e.X, e.Y);
                }

                Dist = Distance(P1, P2);
                for (int i = 0; i < NearestPoints.Length; i++)
                {
                    if (NearestPoints[i] > -1)
                    {
                        P1.X = e.X;
                        P1.Y = NearestPoints[i];
                        P2.X = e.X;
                        P2.Y = e.Y;
                        tmp = Distance(P1, P2);

                        if ((i == 0) || (tmp < Dist))
                        {
                            Dist = tmp;
                            miSeriesIndex = i;
                            double XVal = TeeChart.Series[i].XScreenToValue(P1.X);
                            miValueIndex = (int)Math.Round(XVal, 0);
                        }
                    }
                }

                return true;
            }
            catch
            {
                return false;
            }
        }

 
        #endregion

        #region Helper functions for Trend Control
        //Utility function for the trend control
        private double Distance(Point Pt1, Point Pt2)
        {
            int dx, dy;

            dx = Pt1.X - Pt2.X;
            dy = Pt1.Y - Pt2.Y;

            return Math.Sqrt((dx * dx) + (dy * dy));
        }

        private bool IsCursurInChart(MouseEventArgs e)
        {
            if (e.X < TeeChart.Chart.ChartRect.Left) return false;
            if (e.X > TeeChart.Chart.ChartRect.Right) return false;
            if (e.Y < TeeChart.Chart.ChartRect.Top) return false;
            if (e.Y > TeeChart.Chart.ChartRect.Bottom) return false;
            return true;
        }
        #endregion


    }
}
